@@ -18,8 +18,9 @@
 
 package Lintian::Tag::Standard;
 
-use strict;
+use v5.20;
 use warnings;
+use utf8;
 
 use Moo;
 use namespace::clean;
